
-- Defining Options

local options = { 
    type='group',
    args = {
	debug = {
            type = 'toggle',
            name = 'debug',
            desc = 'Toggles debug mode',
            get = "IsDebug",	
            set = "ToggleDebug",
        },
	--[[config = {
		type = 'toggle',
		name = 'config',
		desc = 'Toggles config mode',
		get = "IsConfig",
		set = "ToggleConfig",
        },]]--
    veEnabled = {
        type = 'toggle',
        name = 'veEnabled',
        desc = 'Should VE be used?',
        get = "IsVeEnabled",
        set = "ToggleVeEnabled",
        },
    swdEnabled = {
        type = 'toggle',
        name = 'swdEnabled',
        desc = 'Should SW:D be used?',
        get = "IsSwdEnabled",
        set = "ToggleSwdEnabled",
        },
    swdCutOff = {
            type = 'range',
            name = 'SWD Cutoff',
            desc = "Minimum hp needed to suggest SWD",
            get = "IsSwdCutOff",
            set = "SetSwdCutOff",
            min = 0,
            max = 1, -- since percentage, max is 100%,
            step = 0.01, -- every 1%
            isPercent = true,
        },
    },
}



-- Calling libraries and instancing them
Darkness = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0")
Darkness:RegisterChatCommand("/Darkness", "/dark", options)
local lego1 = LibStub("LegoBlock-Beta1"):New("SpellToCastFirst" ,"Block1", nil, {})
local lego2 = LibStub("LegoBlock-Beta1"):New("SpellToCastSecond" ,"Block2", nil, {})

-- TimeBlock = DongleStub("Dongle-1.0"):New("Darkness")
-- DarknessLegos = DongleStub("Dongle-1.0"):New("Legos", {})
-- local legoblocks = {}

-- Setting default values to variables
Darkness:RegisterDB("DarknessDB", "DarknessDBPC")
Darkness:RegisterDefaults("profile", {
	debug = false,
--	config = false,
    veEnabled = true,
    swdEnabled = true,
	lego1 = {
		width = 64,
		height = 32,
		showText = true,
		showIcon = true,
		scale = 1,
		group = nil,
		x = 64,
	},
	lego2 = {
		width = 64,
		height = 32,
		showText = true,
		showIcon = true,
		scale = 1,
		group = nil,
		x = 64,
	},
    swdCutOff = 0.5
} )

-- Setting default values to local variables
local vampiricTouchIcon = "Spell_Holy_Stoicism"
local vampiricTouchText = "Vampiric Touch"
-- local vampiricTouchText = "VT"
local shadowWordPainIcon = "Spell_Shadow_ShadowWordPain"
local shadowWordPainText = "Shadow Word: Pain"
-- local shadowWordPainText = "SW:P"
local vampiricEmbraceIcon = "Spell_Shadow_UnsummonBuilding"
local vampiricEmbraceText = "Vampiric Embrace"
local shadowWordDeathText = "Shadow Word: Death"
-- local shadowWordDeathText = "SW:D"
local shadowWordDeathIcon = "Spell_Shadow_DemonicFortitude"
local mindBlastText = "Mind Blast"
-- local mindBlastText = "MB"
local mindBlastIcon = "Spell_Shadow_UnholyFrenzy"
local mindFlayText = "Mind Flay"
-- local mindFlayText = "MF"
local mindFlayIcon = "Spell_Shadow_SiphonMana"
local minimumCooldown = 1.5
local lag = 0.13
local spellCast, spellCast2, spellCast3
local label

local textureTable = {
	["Vampiric Touch"] = "Spell_Holy_Stoicism",
	["Shadow Word: Pain"] = "Spell_Shadow_ShadowWordPain",
	["Vampiric Embrace"] = "Spell_Shadow_UnsummonBuilding",
	["Shadow Word: Death"] = "Spell_Shadow_DemonicFortitude",
	["Mind Blast"] = "Spell_Shadow_UnholyFrenzy",
	["Mind Flay"] = "Spell_Shadow_SiphonMana",
}
	
local spellHistory = {
    [1] = nil,
    [2] = nil,
    [3] = nil,
}

local spellTHistory = {
    [1] = nil,
    [2] = nil,
    [3] = nil,
}

local spellQueue = {
    [1] = nil,
    [2] = nil,
    [3] = nil,
}
    
-- Table to keep track of spells and such	
-- Table for each spell:
-- [Whether it is queued], [Which number it was cast last as (incremented)], [Number of times it's been ignored by the player] 
local stct = {
    ["Vampiric Touch"] = {0,0, 0} ,
    ["Shadow Word: Pain"] = {0,0,0},
    ["Vampiric Embrace"] = {0,0,0},
    ["Shadow Word: Death"] = {0,0,0},
    ["Mind Blast"]= {0,0,0},
    ["Mind Flay"] = {1,0,0},
    }

--The rest
function Darkness:OnInitialize()
    -- Called when the addon is loaded
end

function Darkness:OnEnable()
	if select(2, UnitClass("player")) == "PRIEST" then  
    		self:DebugPrint("Darkness Enabled - Player class is priest")

	        lego1:SetDB(self.db.profile.lego1)
	        lego2:SetDB(self.db.profile.lego2)
            self:UpdateLegoBlocks(vampiricTouchText,shadowWordPainText)  
            self:RegisterEvent("UNIT_SPELLCAST_SENT", "OnSpellcast")
            --self:RegisterEvent("PLAYER_TARGET_CHANGED", "OnTargetChanged")	
            self:RegisterEvent("PLAYER_TARGET_CHANGED")	
            self:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE", "SpellResistCheck")
            --self:RegisterEvent("COMBAT_RATING_UPDATE", "UpdateGCD")
    else
        self:DebugPrint("Darkness not enabled, wrong class");
	end    

end



function Darkness:OnDisable()
    -- Called when the addon is disabled
    self:CancelAllScheduledEvents()
    self:UnregisterAllEvents()
end


function Darkness:FormEvent(label)
    self:Print("FormEvent() called")
    label:SetText("test")
end


function Darkness:OnSpellcast(unit, spell, rank, target)	
    if not UnitIsFriend("player","target") then

	
        self:UpdateSpellHistory(spell)
        self:SetSpellTHistory()
         
        spellQueue[1] = self:SpellToCastT(1,1,_)
        self:UpdateSpellTHistory(spellQueue[1])
        
        spellQueue[2] = self:SpellToCastT(2,2, _)
        self:UpdateSpellTHistory(spellQueue[2])
        
        self:UpdateLegoBlocks(spellQueue[1], spellQueue[2])

        if spell ~= nil then
			-- self:DebugPrint("(" .. spell .. ") -> ".. spellQueue[1] .. " -> " .. spellQueue[2])
		else
			-- self:DebugPrint("( Target Changed ) -> " .. spellQueue[1] .. " -> " .. spellQueue[2])
		end 
        
        self:ClearSpellTHistory()
	end	
	
end

function Darkness:UpdateSpellHistory(spell)
    if spell ~= nil then
        spellHistory[3] = spellHistory[2]
        spellHistory[2] = spellHistory[1]
        spellHistory[1]= spell
    end
end

function Darkness:ClearSpellHistory(spell)
    for x,y in pairs(spellHistory) do
        y = nil
    end
end

function Darkness:ClearSpellTHistory()
    for x,y in pairs(spellTHistory) do
        y = nil
    end
end

function Darkness:IsInHistory(spell)
    local retValue = 0
    if spell ~= nil then
        for x,y in pairs(spellHistory) do
            if y == spell then
                retValue = x
            end
        end
    end
    return retValue
end

function Darkness:SetSpellTHistory()
    spellTHistory[1] = spellHistory[1]
    spellTHistory[2] = spellHistory[2]
    spellTHistory[3] = spellHistory[3]
end

function Darkness:IsInTHistory(spell)
    local retValue = 0
    if spell ~= nil then
        for x,y in pairs(spellTHistory) do
            if y == spell then
                retValue = x
            end
        end
    end
    -- self:DebugPrint("IsInTHistory: retValue: " .. retValue)
    return retValue
end

function Darkness:UpdateSpellTHistory(spell)
    if spell ~= nil then
        spellTHistory[3] = spellTHistory[2]
        spellTHistory[2] = spellTHistory[1]
        spellTHistory[1]= spell
    end
end


function Darkness:UpdateLegoBlocks(spell1, spell2)
    lego1:SetText(spell1)
    lego2:SetText(spell2)
    lego1:SetIcon("Interface\\Icons\\" .. textureTable[spell1])
    lego2:SetIcon("Interface\\Icons\\" .. textureTable[spell2])
end

function Darkness:SetIcon(spellName)
	local spellIcon
	if spellName == vampiricTouchText then
		spellIcon = vampiricTouchIcon
	elseif spellName == shadowWordPainText then
		spellIcon = shadowWordPainIcon
	elseif spellName == vampiricEmbraceText then
		spellIcon = vampiricEmbraceIcon
	elseif spellName == shadowWordDeathText then
		spellIcon = shadowWordDeathIcon
	elseif spellName == mindBlastText then
		spellIcon = mindBlastIcon
	elseif spellName == mindFlayText then
		spellIcon = mindFlayIcon
	end
	return spellIcon
end
	
function Darkness:PLAYER_TARGET_CHANGED()
--function Darkness:OnTargetChanged()
    self:ClearSpellHistory()
    self:ClearSpellTHistory()
	self:OnSpellcast(_,_,_,"target")
end

function Darkness:SpellResistCheck(arg)
    if string.find(arg, "resisted") or string.find(arg, "immune") then
            for x,y in pairs(spellHistory) do
                if string.find(arg, y) then 
                    spellHistory[x] = 0
                    self:DebugPrint(spellHistory[x] .. " was resisted")
                end
            end
    end
        
end

-- Function which finds the spell to cast based on the table setup.
-- 1 seq = 1.5 seconds (1 gcd)
function Darkness:SpellToCastT(seq, no, last)
    local spellToCast = nil
    local hasteRating = GetCombatRating(20)
    minimumCooldown = 1.5/(1+(hasteRating/1570))+lag
    --self:DebugPrint("Hasterating: " .. hasteRating .. " minimumCooldown: " .. minimumCooldown)
    
    local timeToNextCast = minimumCooldown+(minimumCooldown*(seq-1))
    if spellTHistory[1] == mindFlayText then
       timeToNextCast = timeToNextCast + minimumCooldown
    end
    if spellTHistory[2] == mindFlayText and no == 2 then
        timeToNextCast = timeToNextCast + minimumCooldown
    end
    --self:DebugPrint("timeToNextCast: " .. timeToNextCast)
    for spellName,spellTable in pairs(stct) do
        local timeLeft = self:FindADebuff("target", textureTable[spellName])
        if timeLeft == nil then
            timeLeft = 0
        end
        local isInHistory = self:IsInTHistory(spellName)
        if spellName == vampiricTouchText then
            timeLeft = timeLeft - minimumCooldown
        elseif spellName == mindBlastText or spellname == shadowWordDeathText then
            if isInHistory > 2 then 
                isInHistory = 0 
            end
        end
        
        if (self:getCD(spellName) < timeToNextCast) and isInHistory == 0 and (timeLeft < (timeToNextCast)) then
            stct[spellName][1] = 1
        else
            stct[spellName][1] = 0
        end
    
    end
    
    if stct[vampiricTouchText][1] == 1 then  spellToCast = vampiricTouchText
    elseif stct[shadowWordPainText][1] == 1 then spellToCast = shadowWordPainText
    elseif (stct[vampiricEmbraceText][1] == 1 and self.db.profile.veEnabled ) then spellToCast = vampiricEmbraceText
    elseif stct[mindBlastText][1] == 1 then spellToCast = mindBlastText
    elseif (stct[shadowWordDeathText][1] == 1 and self.db.profile.swdEnabled and UnitHealth("player")/UnitHealthMax("player") > self.db.profile.swdCutOff) then spellToCast = shadowWordDeathText
    else spellToCast = mindFlayText
    end
    -- --self:DebugPrint("spellToCast = " .. spellToCast)
    return spellToCast
  
end

function Darkness:getCD(spellName)
    local startTime, duration,_ = GetSpellCooldown(spellName)
    if duration == nil then
        duration = 0
    end
    if duration ~= 0 then
        duration = startTime+duration - GetTime()
    end
    -- self:DebugPrint("CD of " .. spellName .. ": " .. duration)
    return duration
end

--Function to find a debuff cast by the player.
function Darkness:FindADebuff(unit, matchTexture)
		
		-- self:DebugPrint("FindABuff, Unit = " .. unit);
		-- self:DebugPrint("FindABuff, matchTexture = " .. matchTexture);
		
		local debuffIndex = 1
		while 1 do
			local _, _, texture, _, _, duration, timeLeft  =  UnitDebuff(unit, debuffIndex, _);
			
			if texture == nil then
			--	self:DebugPrint("No debuff found on target")
				return timeLeft
			elseif string.match(texture, "Interface\\Icons\\" .. matchTexture) and timeLeft ~= nil then
				return timeLeft
			--else
			--	self:DebugPrint("Debuff found: " .. texture)
			end
			debuffIndex = debuffIndex + 1
		end	
	
end

-- Print Functions
function Darkness:DebugPrint(debugString)
	-- self:Print("DebugPrint called")
	if self.db.profile.debug == true then
		self:Print("DEBUG - " .. debugString)
	end
end


function Darkness:PrintStuff(pMessage, pIcon)
	if self.db.profile.showOnParrot then
		Parrot:ShowMessage(pMessage, "Notification", true, 1, 0, 0, "Daville", 22, "Inherit", "Interface\\Icons\\" .. pIcon)
	end
	if self.db.profile.showOnScreen then
		UIErrorsFrame:AddMessage(pMessage, 1.0, 1.0, 1.0, 5.0)
	end
	if self.db.profile.showInChat then
		self:Print(pMessage)
	end
	if self.db.profile.showOnSCT then
		SCT_MSG_FRAME:AddMessage(pMessage, 1, 0, 0, 1)
	end
end




-- Functions for toggling and setting options
function Darkness:IsDebug()
    return self.db.profile.debug
end

function Darkness:ToggleDebug()
    self.db.profile.debug = not self.db.profile.debug
end

function Darkness:IsVeEnabled()
    return self.db.profile.veEnabled
end

function Darkness:ToggleVeEnabled()
    self.db.profile.veEnabled = not self.db.profile.veEnabled
end

function Darkness:IsSwdEnabled()
    return self.db.profile.swdEnabled
end

function Darkness:ToggleSwdEnabled()
    self.db.profile.swdEnabled = not self.db.profile.swdEnabled
end

function Darkness:ToggleConfig()
	self.db.profile.config = not self.db.profile.config
	if self.db.profile.config then
		self.db.profile.lego1[hidden] = false
		self.db.profile.lego2[hidden] = false
	else 
		self.db.profile.lego1[hidden] = true
		self.db.profile.lego2[hidden] = true
	end
end

function Darkness:IsConfig()
	return self.db.profile.config
end
function Darkness:IsSwdCutOff()
    return self.db.profile.swdCutOff
end
function Darkness:SetSwdCutOff(v)
    self.db.profile.swdCutOff = v
end
    